# -*- cmake -*-
#
# @file OpenRTMConfig.cmake
# @brief cmake-config file for OpenRTM-aist
# @author Noriaki Ando
#
# This file is used for cmake config-mode.
# The following variables are defined.
#
# Basic compiler/linker options
# - OPENRTM_CFLAGS: cflags 
# - OPENRTM_INCLUDE_DIRS: include directories
# - OPENRTM_LDFLAGS: linker options
# - OPENRTM_LIBRARY_DIRS: library directories
# - OPENRTM_LIBRARIES: libraries
# - OPENRTM_LIB_DIR: OpenRTM's lib directory
#
# OpenRTM-aist specific directory
# - COIL_INCLUDE_DIR: coil include dir
# - OPENRTM_INCLUDE_DIR: OpenRTM's include directory
# - OPENRTM_LIB_DIR: OpenRTM's lib directory
# - OPENRTM_DATA_DIR: OpenRTM's shared directory
#
# OpenRTM-aist version
# - OPENRTM_VERSION: x.y.z version
# - OPENRTM_VERSION_MAJOR: major version number
# - OPENRTM_VERSION_MINOR: minor version number
# - OPENRTM_VERSION_PATCH: revision number
# - OPENRTM_SHORT_VERSION: short version number 1.1.0->110
#
# OpenRTM-aist's CORBA related settings
# - OPENRTM_ORB: CORBA implementation
# - OPENRTM_IDL_WRAPPER: rtm-skelwrapper command
# - OPENRTM_IDL_WRAPPER_FLAGS: rtm-skelwrapper flag
# - OPENRTM_IDLC: IDL command
# - OPENRTM_IDLFLAGS: IDL optins

message(STATUS "OpenRTMConfig.cmake ${OpenRTM_VERSION} found.")
message(STATUS "Configrued by configuration mode.")


# Basic compiler/linker options
set(OPENRTM_CFLAGS  -Wall -fPIC -O2)
set(OPENRTM_INCLUDE_DIRS /usr/include /usr/include/coil-2.0 /usr/include/openrtm-2.0 /usr/include/openrtm-2.0/rtm/idl /usr/include/openrtm-2.0/rtm/ext)
set(OPENRTM_LDFLAGS -luuid -lrt -ldl -export-dynamic)
set(OPENRTM_LIBRARY_DIRS /usr/lib/arm-linux-gnueabihf)
set(OPENRTM_LIBRARIES pthread;RTC2;omniDynamic4;omniORB4;omnithread;rtmCamera2;rtmManipulator2)
set(OPENRTM_LIB_DIR /usr/lib/arm-linux-gnueabihf)
set(OPENRTM_INCLUDE_DIR /usr/include/openrtm-2.0)
set(OPENRTM_DIR /usr)

# OpenRTM-aist specific directory
set(COIL_INCLUDE_DIR /usr/include/coil-2.0)
set(OPENRTM_DATA_DIR /usr/share)

# OpenRTM-aist version
set(OPENRTM_VERSION 2.0.2)
set(OPENRTM_VERSION_MAJOR 2)
set(OPENRTM_VERSION_MINOR 0)
set(OPENRTM_VERSION_PATCH 2)
set(OPENRTM_SHORT_VERSION 202)

# OpenRTM-aist's CORBA related settings
set(OPENRTM_ORB omniORB)
set(OPENRTM_IDL_WRAPPER rtm2-skelwrapper)
set(OPENRTM_IDL_WRAPPER_FLAGS --include-dir= --skel-suffix=Skel --stub-suffix=Stub)
set(OPENRTM_IDLC omniidl)
set(OPENRTM_IDLFLAGS -bcxx -Wba -nf -Wbuse_quotes -Wbshortcut -I${OPENRTM_INCLUDE_DIR}/rtm/idl)

if(NOT PKG_CONFIG_FOUND)
  include(CMakeFindDependencyMacro)
  find_dependency(PkgConfig)
endif()
#
# Getting omniORB settings
#
pkg_check_modules(OMNIORB REQUIRED omniORB4)
if(NOT OMNIORB_DIR)
  if(OMNIORB_FOUND)
    set(OMNIORB_DIR )
  endif()
  set(OMNIORB_DIR CACHE PATH omniORB root directory)
endif()


set(OPENRTM_CFLAGS ${OPENRTM_CFLAGS} ${OMNIORB_CFLAGS})
set(OPENRTM_INCLUDE_DIRS ${OPENRTM_INCLUDE_DIRS} ${OMNIORB_INCLUDE_DIRS})
set(OPENRTM_LDFLAGS ${OPENRTM_LDFLAGS} ${OMNIORB_LDFLAGS})
set(OPENRTM_LIBRARY_DIRS ${OPENRTM_LIBRARY_DIRS} ${OMNIORB_LIBRARY_DIRS})


message(STATUS "OpenRTM-aist configuration done")
# end of OpenRTMConfig.cmake
